

import java.io.IOException;
import java.io.InputStream;
import java.util.Iterator;
import org.mindswap.pellet.jena.PelletReasonerFactory;


import com.hp.hpl.jena.query.Query;
import com.hp.hpl.jena.query.QueryExecution;
import com.hp.hpl.jena.query.QueryExecutionFactory;
import com.hp.hpl.jena.query.QueryFactory;
import com.hp.hpl.jena.query.QuerySolution;
import com.hp.hpl.jena.query.ResultSet;
import com.hp.hpl.jena.rdf.model.*;
import com.hp.hpl.jena.reasoner.Reasoner;
import com.hp.hpl.jena.reasoner.ReasonerRegistry;
import com.hp.hpl.jena.reasoner.ValidityReport;
import com.hp.hpl.jena.reasoner.rulesys.GenericRuleReasoner;
import com.hp.hpl.jena.reasoner.rulesys.Rule;
import com.hp.hpl.jena.util.FileManager;


public class HelloSemanticWeb {
	static String defaultNameSpace = "http://org.semwebprogramming/chapter2/people#";
	
	Model _friends = null;
	Model schema = null;
	InfModel inferredFriends = null;
	
	private static String buildQuery(String name){
		String query = "PREFIX owl: <http://www.w3.org/2002/07/owl#> " +
				"PREFIX xsd: <http://www.w3.org/2001/XMLSchema#> " +
				"PREFIX rdfs: <http://www.w3.org/2000/01/rdf-schema#> " +
				"PREFIX rdf: <http://www.w3.org/1999/02/22-rdf-syntax-ns#> " +
				"PREFIX foaf: <http://xmlns.com/foaf/0.1/> " +
				"PREFIX dc: <http://purl.org/dc/elements/1.1/> " +
				"PREFIX : <http://dbpedia.org/resource/> " +
				"PREFIX dbpedia2: <http://dbpedia.org/property/> " +
				"PREFIX dbpedia: <http://dbpedia.org/> " +
				"PREFIX skos: <http://www.w3.org/2004/02/skos/core#> " +
				"PREFIX p: <http://dbpedia.org/property/> " +
				"PREFIX dbo: <http://dbpedia.org/ontology/> " +
				"PREFIX text: <http://jena.apache.org/text#>" +
				"SELECT ?name ?abstract ?object "+
				"{ ?object p:name ?name;" +
				//"  p:name \"" + name + "\"@en ;" +
				"  dbo:abstract ?abstract ." +
				"FILTER (lang(?abstract) = \"en\") . " +
				"FILTER regex(?name,\"^" + name + "$\",\"i\") . " +
				"FILTER (lang(?name) = \"en\") . }";
//				+
//				"WHERE { "+
//				"?object foaf:name \"" + name + "\" . " +
//				"?object foaf:name ?name . " + 
//				"?object dbo:abstract ?abstract . " +
//				"FILTER (lang(?abstract) = \"en\") . " +
//				"FILTER (lang(?name) = \"en\") . " +
//				" }";
		
		return query;
	}

	public static String runQuery(String arg) throws IOException {
	
		String testQuery = "PREFIX owl: <http://www.w3.org/2002/07/owl#> " +
				"PREFIX xsd: <http://www.w3.org/2001/XMLSchema#> " +
				"PREFIX rdfs: <http://www.w3.org/2000/01/rdf-schema#> " +
				"PREFIX rdf: <http://www.w3.org/1999/02/22-rdf-syntax-ns#> " +
				"PREFIX foaf: <http://xmlns.com/foaf/0.1/> " +
				"PREFIX dc: <http://purl.org/dc/elements/1.1/> " +
				"PREFIX : <http://dbpedia.org/resource/> " +
				"PREFIX dbpedia2: <http://dbpedia.org/property/> " +
				"PREFIX dbpedia: <http://dbpedia.org/> " +
				"PREFIX skos: <http://www.w3.org/2004/02/skos/core#> " +
				"PREFIX p: <http://dbpedia.org/property/> "+
				"PREFIX dbo: <http://dbpedia.org/ontology/> "+
				"SELECT DISTINCT ?name ?birth ?death ?person ?abstract "+
				"WHERE { "+
				"?person dbo:birthPlace :Berlin . " + 
				"?person dbo:birthDate ?birth . " + 
				"?person p:name ?name . " +
				"?person dbo:deathDate ?death . " +
				"?person dbo:abstract ?abstract . " +
				"FILTER (?birth < \"1900-01-01\"^^xsd:date) . " +
				"FILTER (lang(?abstract) = \"en\") . " +
				"FILTER (lang(?name) = \"en\") . " +
				" }";
		
		String queryString = buildQuery(arg); 

		// now creating query object
		Query query = QueryFactory.create(queryString);
		// initializing queryExecution factory with remote service.
		// **this actually was the main problem I couldn't figure out.**
		QueryExecution qexec = QueryExecutionFactory.sparqlService("http://dbpedia.org/sparql", query);

		//after it goes standard query execution and result processing which can
		// be found in almost any Jena/SPARQL tutorial.
		try {
		    ResultSet results = qexec.execSelect();
		    String string = "";
		    String tstr = "";
		    int i = 0;
		    
		    while(results.hasNext()) {
    			QuerySolution qs = results.next();
    			RDFNode node = qs.get("name");
    			tstr = node.asLiteral().toString();
    			string += "\n\n" + tstr.substring(0, tstr.indexOf('@'));
    			//System.out.println(string);
//    			System.out.print(i++ + " : " + string.substring(0, string.length() - 3));
    			node = qs.get("abstract");
    			string += "\n";
    			tstr = node.asLiteral().toString();
    			string += tstr.substring(0, tstr.indexOf('@'));
    			//System.out.print(string);
//    			node = qs.get("death");
//    			string = node.asLiteral().toString();
//    			System.out.print(", died " + string.substring(0, string.indexOf("^^")) + "\n");
//    			node = qs.get("abstract");
//    			string = node.asLiteral().toString();
//    			System.out.print(string.substring(0, string.length() - 3) + "\n\n");
		    }
		    return string;
		}
		finally {
		   qexec.close();
		}
		
		
	}

}
